/** An instance represents one duckpin bowling game */
public class Game {
    private String name= "";  // The person playing the game
    // The ten frames are in frames[1..10].
    // frames[0] is not used. frames[11..12] are null
    private Frame[] frames= new Frame[13]; 
    
    /** Constructor: a particular game: three 3-roll frames, 3 spares, 3 strikes,
                     and a tenth frame with strike, 5, 5. Name is "TEST" */
    public Game(){
        name= "TEST";
        frames[1]= new Frame(3, 4, 1, false);
        frames[2]= new Frame(9, 0, 1, false);
        frames[3]= new Frame(0, 0,10, false);
        frames[4]= new Frame(7);
        frames[5]= new Frame(9);
        frames[6]= new Frame(0);
        frames[7]= new Frame();
        frames[8]= new Frame();
        frames[9]= new Frame();
        frames[10]= new Frame(10, 5, 5, true);
    }
    
    /** Constructor: a game for person t, with balls rolled given by String s.
        s contains a list of integers --the scores on each ball, in turn--
        separated by at least one blank */
    public Game(String t, String s) {
        name= t;
        StringBuffer sb= new StringBuffer(s);
        int i= 1;
        // invariant: frames[1..i-1] has been calculated, and 
        //            sb contains the balls rolled in frames i..10
        for (i= 1; i != 10; i= i+1) {
            frames[i]= getFrame9(sb);   
        } 
        
        frames[i]= getFrame10(sb);
        frames[11]= null; frames[12]= null;
    }
    
    /** = name of person */
    public String getName() {
        return name;
    }
    
    // = the array of frames for the course
    public Frame[] getFrames() {
        return frames;
    }
    
    /** = a Frame (for anything but the tenth frame), reading the
          balls rolled in the frame from sb and deleting them from sb */
    private Frame getFrame9(StringBuffer sb) {
        int firstBall= stripInt(sb);
        if (firstBall == 10) {
            return new Frame();
        }
        int secondBall= stripInt(sb);
        if (firstBall + secondBall == 10) {
            return new Frame(firstBall);
        }
        return new Frame(firstBall, secondBall, stripInt(sb), false);
    }
    
    /** =  a Frame for the tenth frame, reading the balls rolled
           in the frame from sb and deleting them from sb */
    private Frame getFrame10(StringBuffer sb) {
        return new Frame(stripInt(sb), stripInt(sb), stripInt(sb), true);
    }
    
    /** Strip off the next integer in sb --which must exist-- 
        and return it. For example, if sb contains the string<br>
        
        "   10 9   4   8  3", then change it to<br>
        " 9   4   8  3" and return the value 10.<br>
        
        Note that a blank need not follow the last integer, but it may.*/
    public int stripInt(StringBuffer sb) {
        // remove preceding blanks from sb
        int k= 0;
        while (sb.charAt(k) == ' ') {
            k= k+1;
        }
        sb.delete(0, k);
    
        // Find position k of first blank of sb 
        //    --set k to sb.length() if sb does not contain a blank
        k= 0;
        while (k != sb.length()  &&  sb.charAt(k) != ' ') {
            k= k+1;
        }
        
        String res= sb.substring(0,k);
        sb.delete(0,k);
        
        return Integer.parseInt(res);
    }
    
    /** = String representation of the game. This is the f.card(i) (which is four
          characters long) for each frame f of the game, with a blank character
          SEPARATING each pair. Using # to denote a blank character, here is an example:
          For the game with rolls
                 (1,3,5),(2,8),(1,4,5),(0,0,10),(10),(0,1,9),(10),(10), (10), (0,0,10)
          produce the string:
                 "135##2/###145##0010#X####019##X####X####X####0010"
      */
    public String toString() {
        String rep= "";
        for (int i= 1; i != 11; i= i+1) {
            if (i != 1)
                rep= rep + " ";
            rep= rep + frames[i].card();
        }
        
        return rep;
    }
    
    /** = the score for the game (i.e. the sum of the scores in the individual 10 frames)*/
    public int score() {
        int total= 0;
        for (int i= 1; i != 11; i= i+1) {
            total= total + frames[i].score(frames[i+1], frames[i+2]);
        }
        
        return total;
    }
    
    /** = the score card for the game. This is the accumulated score in each
          frame. For example, for the second frame, give the sum of the scores
          in the first and second frame. For each frame, the score should be
          exactly three digits long --prepend blank characters to the score,
          if necessary, to make the score 3 chars long. Further, SEPARATE adjacent
          scores by a TWO blanks. For example, using # to denote a blank, here is a
          possible score:
          #30###60###90##120##150##180##210##240##270##300
          */
    public String card() {
        String res= "";
        int total= 0;
        // total is the score for frames 0..i-1 and
        // res is the score card for frames 0..i-1
        for (int i= 1; i != 11; i= i+1) {
            int sc= frames[i].score(frames[i+1], frames[i+2]);
            total= total + sc;
            if (i != 1) {
              res= res + "  "; 
            }
            res= res + prepend("" + total);
        }
        return res;
    }
    
    /** = s, but prepended with blanks to make it 3 chars long it
          it is less than 3 chars long */
    private static String prepend(String s) {
        while (s.length() < 3) {
            s= " " + s;
        }
        return s;
    }
         
}